# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ..core.pydantic_utilities import UniversalBaseModel
import typing
import typing_extensions
from ..core.serialization import FieldMetadata
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .llm_as_judge_code import LlmAsJudgeCode


class Base(UniversalBaseModel):
    id: typing.Optional[str] = None
    project_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="projectId")
    ] = None
    name: str
    sampling_rate: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="samplingRate")
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")
    ] = None
    created_by: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="createdBy")
    ] = None
    last_updated_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="lastUpdatedAt")
    ] = None
    last_updated_by: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="lastUpdatedBy")
    ] = None
    action: typing.Optional[typing.Literal["evaluator"]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AutomationRuleEvaluator_LlmAsJudge(Base):
    type: typing.Literal["llm_as_judge"] = "llm_as_judge"
    code: typing.Optional[LlmAsJudgeCode] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AutomationRuleEvaluator = AutomationRuleEvaluator_LlmAsJudge
